﻿--[[
Name: LibSpecialEvents-Mount-4.0
Revision: $Revision: 57 $
Author: Tekkub Stoutwrithe (tekkub@gmail.com)
Website: http://www.wowace.com/
Description: Special events for mounting
Dependencies: LibStub, CallbackHandler-1.0, LibSpecialEvents-Aura-4.0
]]

local vmajor, vminor = "LibSpecialEvents-Mount-4.0", "$Rev: 57 $";

if( not LibStub( "LibSpecialEvents-Aura-4.0", true ) ) then
    error( vmajor .. " requires LibSpecialEvents-Aura-4.0" );
    return;
end--if

if( not LibStub( "LibGratuity-3.0", true ) ) then
    error( vmajor .. " requires LibGratuity-3.0" );
    return;
end--if

local lib, oldMinor = LibStub:NewLibrary( vmajor, vminor );
if( not lib ) then
    return;
end--if

local mountSpeed, flightSpeed;

do
    local locale = GetLocale();
    
    mountSpeed = ( ( locale == "deDE" ) and "Erh\195\182\ht Tempo um (.+)%%" ) or
        ( ( locale == "koKR" ) and "이동 속도 ([60|100]+)%%만큼 증가" ) or
        ( ( locale == "frFR" ) and "Augmente la vitesse de (%d+)%%" ) or
        ( ( locale == "esES" ) and "Aumenta la velocidad en un (.+)%%" ) or
        ( ( locale == "zhCN" ) and "^速度提高(.+)%%" ) or
        ( ( locale == "zhTW" ) and "速度提高(.+)%%" ) or
        "Increases speed by (.+)%%";

    flightSpeed = ( ( locale == "esES" ) and "Aumenta la velocidad de vuelo en un (.+)%%" ) or
        ( ( locale == "deDE" ) and "Fluggeschwindigkeit um (.+)%% erh\195\182ht." ) or
        ( ( locale == "frFR" ) and "Augmente la vitesse de vol de (.+)%%" ) or
        ( ( locale == "koKR" ) and "비행 속도 (.+)%%만큼 증가" ) or
        ( ( locale == "zhCN" ) and "^飞行速度提高(.+)%%" ) or
        ( ( locale == "zhTW" ) and "飛行速度提高(.+)%%" ) or
        "Increases flight speed by (.+)%%";
end--do

if( not lib.vars ) then
    lib.vars = {};
end--if

for buff, i in LibStub( "LibSpecialEvents-Aura-4.0" ):BuffIter( "player" ) do
    lib:PlayerBuffGained( nil, buff, i );
end--for

local CallbackHandler = LibStub:GetLibrary( "CallbackHandler-1.0" );

if( lib.callbacks ) then
    lib:UnregisterAll( lib );
    lib:UnhookDialogs();
else
    lib.callbacks = LibStub( "CallbackHandler-1.0" ):New( lib, nil, nil, "UnregisterAll" );
end--if

function lib:PlayerBuffGained( _, buff, i )
    if( UnitOnTaxi( "player" ) or self.vars.mounted ) then
        return;
    end--if

    local gratuity = LibStub( "LibGratuity-3.0" );

    gratuity:SetUnitBuff( "player", i );

    local txt = gratuity:GetLine( 2 );

    if( not txt ) then
        return;
    end--if

    local speed = txt:match( mountSpeed );

    if( speed ) then
        self.vars.mounted, self.vars.mountspeed, self.vars.flying = buff, speed, false;

        lib.callbacks:Fire( "Mounted", buff, speed, i, false );
    else
        speed = txt:match( flightSpeed );
        if(  speed and ( not self.vars.mounted ) ) then
            self.vars.mounted, self.vars.mountspeed, self.vars.flying = buff, speed, true;
            lib.callbacks:Fire( "Mounted", buff, speed, i, true );
        end--if
    end--if
end--lib:PlayerBuffGained( _, buff, i )


function lib:PlayerBuffLost( _, buff )
    if( buff == self.vars.mounted ) then
        lib.callbacks:Fire( "Dismounted", buff, self.vars.mountspeed, self.vars.flying );
        self.vars.mounted, self.vars.mountspeed, self.vars.flying = nil, nil, nil;
    end--if
end--lib:PlayerBuffLost( _, buff )


-----------------------------
--      Query Methods      --
-----------------------------

function lib:PlayerMounted()
    return self.vars.mounted, self.vars.mountspeed;
end--lib:PlayerMounted()

function lib:PlayerFlying()
    return self.vars.flying;
end--lib:PlayerFlying()

LibStub( "LibSpecialEvents-Aura-4.0" ).RegisterCallback( lib, "PlayerBuffGained" );
LibStub( "LibSpecialEvents-Aura-4.0" ).RegisterCallback( lib, "PlayerBuffLost" );
